
#import "GameManager.h"
#import "MainView.h"

int W=320;
int H=480;

id key;
id nothing;
int points;
int state; 

@implementation GameManager

#pragma mark ============================= Init Methods ===============================

+ (GameManager*) getInstance  {
    
    static GameManager* gameManager;
		
	if (!gameManager) {
        gameManager = [[GameManager alloc] init];
        NSLog(@"gameManager Singleton angelegt!");
		[gameManager preloader];            
	}
	    
    return gameManager;
}

- (void) preloader {
    sprites = [[NSMutableArray alloc] initWithCapacity:20];
    newSprites = [[NSMutableArray alloc] initWithCapacity:20];
    destroyableSprites = [[NSMutableArray alloc] initWithCapacity:20];
    
    background = [self getPic: @"background.png"];
    
    state = LOAD_GAME;
}

- (void) loadGame { 
    
    [sprites removeAllObjects];
    [newSprites removeAllObjects];
    [destroyableSprites removeAllObjects];      
    
    highscoresKey = @"highscoresKey";      
    highscores = [self readObjectforKey: highscoresKey];
    if (!highscores) {
        highscores = [[NSMutableDictionary alloc] initWithCapacity: 20];
    }
    
    saved = 0;
    lost = 0;
    savedMax = 20;
        
    
    for (int i=0; i<100; i+=25) {
        [self createSprite:ZOMBIE 
                     speed:CGPointMake(0, 0) 
                       pos:CGPointMake(-20, H/2-50+i)];
    }
    
    /*
     SPALTE 1: 62px
     SPALTE 2: 114px
     SPALTE 3: 166px
     SPALTE 4: 218px
     */
    
    //Autos Norm
    CarDrawer *drawer = [[CarDrawer alloc] init];
    [drawer drawer];
    
}

- (void) createSprite: (int) type 
                speed: (CGPoint) sxy 
                  pos: (CGPoint) pxy {    
    
    if (type == PLAYER) { 
        player = [[Player alloc] initWithPic: [self imageRnd] 
                                    frameCnt: 4 
                                   frameStep: 3
                                       speed: sxy
                                         pos: pxy];     
        [player setType: PLAYER];  
        [player setHealth: 25];
        [newSprites addObject: player];
        [player release];
    } else if (type == ZOMBIE) {
        Zombie *zombie = [[Zombie alloc] initWithPic: @"zombie_4f.png" 
                                            frameCnt: 4 
                                           frameStep: 3
                                               speed: sxy
                                                 pos: pxy]; 
        
        [zombie setType: ZOMBIE];
        [newSprites addObject: zombie];
        [zombie release];
        
    } else if (type == CAR) {
        Car *car = [[Car alloc] initWithPic: @"car_black.png"
                                   frameCnt: 2 
                                  frameStep: 5
                                      speed: sxy
                                        pos: pxy];
        
        [car setType: CAR];
        [newSprites addObject: car];
        [car release];  
    } else if (type == CAR_180) {
        Car_180 *car = [[Car_180 alloc] initWithPic: @"car_black_180°.png"
                                   frameCnt: 2 
                                  frameStep: 5
                                      speed: sxy
                                        pos: pxy];
        
        [car setType: CAR_180];
        [newSprites addObject: car];
        [car release];  
    } else if (type == ANIMATION) {        
        Animation *ani = [[Animation alloc] initWithPic: @"smoke_6f.png"
                                               frameCnt: 6 
                                              frameStep: 3
                                                  speed: sxy
                                                    pos: pxy];        
        [ani setType: ANIMATION];
        [newSprites addObject: ani];
        [ani release];         
    } else {
        NSLog(@"ERROR: Unbekannter Sprite-Typ: %i", type);
    }    
}

- (void) createExplosionFor: (Sprite *) sprite { 
    CGPoint p = [Animation getOriginBasedOnCenterOf: [sprite getRect] 
                                             andPic: @"smoke_6f.png"
                                       withFrameCnt: 6]; 
    
    
    [self createSprite: ANIMATION 
                 speed: CGPointMake(0, 0) 
                   pos: p];        
}

#pragma mark ============================= Game Handler ===============================

- (void) touchBegan: (CGPoint) p {    
    [self handleStates];
    if (state == PLAY_GAME) {
        for (Sprite *sprite in sprites) { 
            if ([sprite isActive]) {             
                if ([sprite getType] == ZOMBIE || [sprite getType] == PLAYER) {
                    [(Zombie *) sprite setTouch: p];
                } 
            }   
        } 
    }    
}

- (void) touchMoved: (CGPoint) p {
    if (state == PLAY_GAME) {
        [self touchBegan: p];
    }    
}

- (void) touchEnded {
    if (state == PLAY_GAME) {
        for (Sprite *sprite in sprites) { 
            if ([sprite isActive]) {             
                if ([sprite getType] == ZOMBIE || [sprite getType] == PLAYER) {
                    [(Zombie *) sprite touchEnded];
                } 
            }   
        } 
    }    
}

- (void) handleStates {  
    
    if (state == GAME_WON) {
        state = LOAD_GAME;
    }
}

- (void) drawStatesWithFrame: (CGRect) frame { 
    W = frame.size.width;
    H = frame.size.height;
    switch (state) {
        case LOAD_GAME: 
            ty = -10;
            tx = -10;
            [self loadGame];
            state = START_GAME;
            
            break;
        case START_GAME:
            
            if (tx >= 0 && tx <= W && ty >= H/2+1 && ty <= H && state == START_GAME) {
                NSLog(@"Highscore");
                state = HIGHSCORES;
                tx = -10;
                ty = -10;
            }
            
            if (tx >= 0 && tx <= W && ty >= 0 && ty <= H/2 && state == START_GAME) {
                NSLog(@"Start Game");
                state = PLAY_GAME;
            }
            
            [background drawAtPoint: CGPointMake(0, 0)];
            [self drawString:@"Start Game with a Tap!" at:CGPointMake(5, 5)];
            
            saved = 0;
            lost = 0;

            
            break;    
        case PLAY_GAME:
            [self playGame];
            break;
        case GAME_OVER:
            [background drawAtPoint: CGPointMake(0, 0)];
            [self drawString: @"G A M E  OVER" at: CGPointMake(5, 5)];
            [self drawString: @"You are dead again." at: CGPointMake(5, 25)];
            [self drawString: [NSString stringWithFormat:@"Saved: %i", saved]
                          at: CGPointMake(5, 45)];
            [self drawString: [NSString stringWithFormat:@"Lost: %i", lost]
                          at: CGPointMake(5, 65)];
            [self drawString: @"Tap screen!" at: CGPointMake(5, 85)];
            
            if (points > [score intValue]) {
                NSNumber *number = [NSNumber numberWithInt:points];
                [highscores setObject:number forKey:dateString];
                [self saveObject: highscores key: highscoresKey];
                
            } else {
                NSLog(@"Score: %i. Leider kein neuer Highscore.", points);
                points = 0;
            }
            
            break;  
            
            
        case HIGHSCORES:
            [background drawAtPoint: CGPointMake(0, 0)];
            [self drawString:@"Your Highscores!" at:CGPointMake(5, 5)];
            
            //Scores lesen
            keys = [highscores keysSortedByValueUsingSelector: @selector(compare:)];
            score = [NSNumber numberWithInt: 0];
            int zeilen = [keys count]*20+45;
            int zeilenCount = [keys count];
            for (key in keys) {
                zeilen-=20;
                score = [highscores objectForKey: key];
                //NSLog(@"Date: %@, Score: %@", key, score);
                [self drawString:[NSString stringWithFormat:@"%i. --- %@ Points", zeilenCount,score] at:CGPointMake(5, zeilen)];
                zeilenCount -=1;
            }
            if (tx >= 0 && tx <= W && ty >= H-H/2 && ty <= H) {
                state = LOAD_GAME;
            }
            if (tx >= 0 && tx <= W && ty >= 0 && ty <= H/2) {
                //Next Code
            }
            
            break;
            
            
        case GAME_WON:
            
            points = 0;
            
            break; 
            
        default: NSLog(@"ERROR: Unbekannter Spielzustand: %i", state);
            break;
    }    
}	 

- (void) savedZombie {
    saved++;
    NSLog(@"%i", saved);
    if (saved >= 4) {
        for (int i=0; i<100; i+=25) {
            [self createSprite:ZOMBIE 
                         speed:CGPointMake(0, 0) 
                           pos:CGPointMake(-20, H/2-50+i)];
        }
        lost = 0;
        saved = 0;
        points += 4;
    }
    if (saved == 3 && lost == 1) {
        for (int i=0; i<99; i+=33) {
            [self createSprite:ZOMBIE 
                         speed:CGPointMake(0, 0) 
                           pos:CGPointMake(-20, H/2- 49+i)];
        }
        lost = 1;
        saved = 0;
        points += 3;
    }
    if (saved == 2 && lost == 2) {
        for (int i=0; i<100; i+=50) {
            [self createSprite:ZOMBIE 
                         speed:CGPointMake(0, 0) 
                           pos:CGPointMake(-20, H/2- 50+i)];
        }
        lost = 2;
        saved = 0;
        points += 2;
    }
    if (saved == 1 && lost == 3) {
        for (int i=0; i<100; i+=100) {
            [self createSprite:ZOMBIE 
                         speed:CGPointMake(0, 0) 
                           pos:CGPointMake(-20, H/2)];
        }
        lost = 3;
        saved = 0;
        points += 1;
    }
    
}

- (void) lostZombie {
    lost++;
    if (lost == 4) {
        [self save];
        [self setState:GAME_WON];
    }
    if (saved == 3 && lost == 1) {
        for (int i=0; i<99; i+= 33) {
            [self createSprite:ZOMBIE 
                         speed:CGPointMake(0, 0) 
                           pos:CGPointMake(-20, H/2- 42+i)];
        }
        lost = 1;
        saved = 0;
        points += 3;
    }
    if (saved == 2 && lost == 2) {
        for (int i=0; i<100; i+=50) {
            [self createSprite:ZOMBIE 
                         speed:CGPointMake(0, 0) 
                           pos:CGPointMake(-20, H/2- 50+i)];
        }
        lost = 2;
        saved = 0;
        points += 2;
    }
    if (saved == 1 && lost == 3) {
        for (int i=0; i<100; i+=100) {
            [self createSprite:ZOMBIE 
                         speed:CGPointMake(0, 0) 
                           pos:CGPointMake(-20, H/2)];
        }
        lost = 3;
        saved = 0;
        points += 1;
    }
}

- (void) playGame {   
    
    [background drawAtPoint: CGPointMake(0, 0)];
    [self manageSprites];             
    [self renderSprites];
    [self drawTargetLine];
    
    NSString *hud  = [NSString stringWithFormat: @"HLT: %i", [player getHealth]];
    NSString *hud2  = [NSString stringWithFormat: @"PTS: %i", points];
    
    
    [self drawString: hud at: CGPointMake(5, 5)];
    [self drawString: hud2 at: CGPointMake(200, 5)];

} 

- (void) checkSprite: (Sprite *) sprite {
    if ([sprite getType] == ZOMBIE || [sprite getType] == PLAYER) {                
        for (Sprite *sprite2test in sprites) {        
            if ([sprite2test getType] == CAR || [sprite2test getType] == CAR_180) {              
                if ([sprite checkColWithSprite: sprite2test]) {                
                    [sprite hit];
                }
            }
        } 
    }  
}


#pragma mark ============================= Helper Methods ===============================

- (void) setState: (int) stt {
    state = stt;
}

- (void) manageSprites {
    //NSLog(@"Sprites: %i destroyable: %i new: %i", [sprites count], [destroyableSprites count], [newSprites count]);
    
    //Cleanup 
    for (Sprite *destroyableSprite in destroyableSprites) { 
        for (Sprite *sprite in sprites) { 
            if (destroyableSprite == sprite) { 
                [sprites removeObject: sprite];
                break;
            }
        }   
    }  
    
    //Neue Sprites hinzufügen 
    for (Sprite *newSprite in newSprites){ 
        [sprites addObject: newSprite];   
    } 
    
    [destroyableSprites removeAllObjects]; 
    [newSprites removeAllObjects];
}

- (void) renderSprites {
    for (Sprite *sprite in sprites) { 
        if ([sprite isActive]) {             
            [self checkSprite: sprite];
            [sprite draw]; 
        } else {
            [destroyableSprites addObject: sprite]; 
        }    
    } 
}

- (NSMutableDictionary *) getDictionary {
	if (!dictionary) { //Hashtable
		dictionary = [[NSMutableDictionary alloc] init]; 
		NSLog(@"Dictionary angelegt!");
	}
	return dictionary;
}

- (UIImage *) getPic: (NSString*) picName {
	@try { 
		UIImage *pic = [[self getDictionary] objectForKey: picName];
		if (!pic) {
			pic = [UIImage imageNamed: picName]; 
			[[self getDictionary] setObject: pic forKey: picName];
            int memory = pic.size.width*pic.size.height*4;
			NSLog(@"%@ gespeichert, Size: %i KB", picName, memory/1024);
            [pic release]; 
		}          
		return pic;
	} 
	@catch (id theException) {
		NSLog(@"ERROR: %@ not found!", picName);		
	} 
	return nil;
}

- (int) getRndBetween: (int) bottom and: (int) top {		
	int rnd = bottom + (arc4random() % (top+1-bottom)); 
	return rnd;
} 

- (void) drawString: (NSString *)str at: (CGPoint) p {
    UIFont *uif = [UIFont fontWithName: @"Verdana-Italic" size: 20];
	CGContextSetRGBFillColor(UIGraphicsGetCurrentContext(), 1, 1, 1, 1);
    [str drawAtPoint: p withFont: uif]; 
} 

- (NSString*)imageRnd {
    int rnd = [self getRndBetween:1 and:2];
    if (rnd == 1) {
        NSString* image = @"car_police.png";
        NSLog(@"%@", image);
        return image;
    } else if (rnd == 2) {
        NSString* image = @"car_black.png";
        NSLog(@"%@", image);
        return image;
    }
}
- (NSString*)imageRnd_180 {
    int rnd = [self getRndBetween:1 and:2];
    if (rnd == 1) {
        NSString* image1 = @"car_police_180.png";
        NSLog(@"%@", image1);
        return image1;
    } else if (rnd == 2) {
        NSString* image2 = @"car_black_180°.png";
        NSLog(@"%@", image2);
        return image2;
    }
}


- (int)getTargetX {
    return 264;
}

- (void) drawTargetLine {
    CGContextRef gc = UIGraphicsGetCurrentContext();
    CGContextSetRGBStrokeColor(gc, 0, 0, 0, 0);
	CGContextSetLineWidth(gc, 5);       
	CGContextMoveToPoint(gc, [self getTargetX], 0); 
	CGContextAddLineToPoint(gc, [self getTargetX], H);
 	CGContextDrawPath(gc, kCGPathStroke);
}

- (id)readObjectforKey:(NSString *)key {
    NSUserDefaults *storage = [NSUserDefaults standardUserDefaults];
    return [storage objectForKey:key];
}
- (void) saveObject: (id) object key: (NSString *) key {
    NSUserDefaults *storage = [NSUserDefaults standardUserDefaults];
    [storage setObject: object forKey: key];
}
- (void)saveScore {
    [background drawAtPoint: CGPointMake(0, 0)];
    [self drawString:@"The Game is Over!" at:CGPointMake(5, 5)];
    
    //lokalisierter Date-String   
    date = [NSDate date];    
    dateFormatter = [[NSDateFormatter alloc] init];  
    [dateFormatter setDateStyle: NSDateFormatterShortStyle];
    [dateFormatter setTimeStyle: NSDateFormatterShortStyle];
    dateString = [dateFormatter stringFromDate: date];
    
    highscoresKey = @"highscoresKey";      
    highscores = [self readObjectforKey: highscoresKey];
    if (!highscores) {
        highscores = [[NSMutableDictionary alloc] initWithCapacity: 20];
    } 
}
- (NSNumber*) changeNumber  {
    
    static NSNumber* number;
    
	if (!number) {
        number = [NSNumber numberWithInt: points];
        NSLog(@"New Number Singleton angelegt!");
	}
    
    return number;
}
/*--------------------------------------------------------------
 --------------------------------------------------------------*/
- (void)save {
    
}
/*--------------------------------------------------------------
 --------------------------------------------------------------*/

- (void) dealloc {        
    [sprites release];
    [newSprites release];
    [destroyableSprites release];
    [[self getDictionary] release];
    [super dealloc];
}

@end
